* Output options: all can be turned on/off by adding or removing the NO;
* page number, date, centering, or page breaks, page length and width;
OPTIONS NOnumber NOdate NOcenter FormDlim=' ' PageSize=MAX LineSize=MAX;
* Eliminate SAS default titles and names of tables in output (TRACE ON to show);
TITLE; ODS TRACE OFF; ODS LISTING;

***********************************************************************************;
*******   MACRO PROGRAMS TO AUTOMATE CALCULATIONS FOR EXAMPLE 6 MODELS      *******;
*******               NOTHING IN HERE NEEDS TO BE CHANGED                   *******;
***********************************************************************************;

* To use FitTest macro;
* FitFewer = Name of ODS InfoCrit table for nested model;
* FitMore  = Name of ODS InfoCrit table for comparison model;
%MACRO FitTest(FitFewer=work., FitMore=work.);
DATA &FitFewer.; LENGTH Name $30.; SET &FitFewer.; Name="&FitFewer."; RUN;
DATA &FitMore.;  LENGTH Name $30.; SET &FitMore.;  Name="&FitMore.";  RUN;
DATA FitCompare; LENGTH Name $30.; SET &FitFewer. &FitMore.; RUN;
DATA FitCompare; SET FitCompare; DevDiff=Lag1(Neg2LogLike)-Neg2LogLike;
     DFdiff=Parms-LAG1(Parms); Pvalue=1-PROBCHI(DevDiff,DFdiff);
     DROP AICC HQIC CAIC; RUN;
TITLE9 "Likelihood Ratio Test for &FitFewer. vs. &FitMore.";
PROC PRINT NOOBS DATA=FitCompare; RUN; TITLE9;
%MEND FitTest;

* To use TotalR2 macro;
* DV =        Case-sensitive name of dependent variable;
* PredFewer = Name of OUTPM=work.= data file of predicted outcomes for nested model;
* PredMore =  Name of OUTPM=work.= data file of predicted outcomes for comparison model;
%MACRO TotalR2(DV=, PredFewer=work., PredMore=work.);
PROC CORR NOPRINT NOSIMPLE DATA=&PredFewer. OUTP=CorrFewer; VAR pred &DV.; RUN;
PROC CORR NOPRINT NOSIMPLE DATA=&PredMore.  OUTP=CorrMore;  VAR pred &DV.; RUN;
DATA CorrFewer; LENGTH Name $30.; SET CorrFewer; Name="&PredFewer."; RUN;
DATA CorrMore;  LENGTH Name $30.; SET CorrMore;  Name="&PredMore.";  RUN;
DATA CorrCompare; LENGTH Name $30.; SET CorrFewer CorrMore; 
     PredCorr=Pred; TotalR2=PredCorr*PredCorr; 
     IF _NAME_="Pred" OR MISSING(_NAME_)=1 THEN DELETE; DROP Pred; RUN;
DATA CorrCompare; SET CorrCompare; TotalR2Diff=TotalR2-LAG1(TotalR2);
     KEEP Name PredCorr TotalR2 TotalR2Diff; RUN; 
TITLE9 "Total R2 (% Reduction) for &PredFewer. vs. &PredMore.";
PROC PRINT NOOBS DATA=CorrCompare; RUN; TITLE9; 
%MEND TotalR2;

* To use PseudoR2 macro;
* Ncov =     TOTAL # entries in covariance parameter estimates table;
* CovFewer = Name of ODS CovParms table for nested model;
* CovMore =  Name of ODS CovParms table for comparison model;
%MACRO PseudoR2(NCov=, CovFewer=work., CovMore=work.);
DATA &CovFewer.; LENGTH Name $30.; SET &CovFewer.; Name="&CovFewer."; RUN;
DATA &CovMore.;  LENGTH Name $30.; SET &CovMore.;  Name="&CovMore.";  RUN;
DATA CovCompare; LENGTH Name $30.; SET &CovFewer. &CovMore.; RUN;
DATA CovCompare; SET CovCompare; 
     PseudoR2=(LAG&Ncov.(Estimate)-Estimate)/LAG&Ncov.(Estimate); RUN;
DATA CovCompare; SET CovCompare; 
     IF CovParm IN("UN(2,1)","UN(3,1)","UN(4,1)","UN(3,2)","UN(4,2)","UN(4,3)") 
     THEN DELETE; RUN;
TITLE9 "PsuedoR2 (% Reduction) for &CovFewer. vs. &CovMore.";
PROC PRINT NOOBS DATA=CovCompare; RUN; TITLE9;
%MEND PseudoR2;


***********************************************************************************;
*******     BEGIN DATA MANIPULATION FOR EXAMPLE 6 THREE-LEVEL MODELS        *******;
*******                 CHANGE "filesave" to your directory                 *******;
***********************************************************************************;

* Defining global variable for file location to be replaced in code below;
%LET filesave= C:\Dropbox\23_PSQF7375_AdvLong\PSQF7375_AdvLong_Example6;
* Location for SAS files for these models (uses macro variable filesave);
LIBNAME filesave "&filesave.";

* Import data into work library;
DATA work.Example6; SET filesave.AdvLong_Example6; RUN;

* Create pair-level (between-family) age at wave1;
PROC SORT DATA=work.Example6; BY PairID TwinID Wave; RUN;
PROC MEANS NOPRINT DATA=work.Example6; BY PairID; VAR agew1;
     OUTPUT OUT=work.Age MEAN(agew1)=PairAge; RUN;

* Create predictors;
DATA work.Example6; MERGE work.Example6 work.Age; BY PairID;
* Center pair-level (between-family) at 85 for use at level 3;
  PairAge85 = PairAge - 85; LABEL PairAge85= "PairAge85: Pair Mean Age at Wave1 (0=85)";
* Within-person center age for use at level-1 (VARIABLE CENTERING);
  time = age - agew1;   LABEL time= "time: Time Since Wave1 (0= Age at Wave1)";
* Make centered and string versions of zygosity for easier output reading;
  IF zygosity=1 THEN DO; IsDZ=0; zyg="MZ"; END;
  IF zygosity=2 THEN DO; IsDZ=1; zyg="DZ"; END;
  LABEL IsDZ= "IsDZ: Zygosity (0=MZ, 1=DZ)";
* Select only cases with complete data per occasion;
  IF NMISS(info, time, PairAge, IsDZ)>0 THEN DELETE; 
* Remove last occasion that is mostly missing data;
  IF Wave=5 THEN DELETE;
RUN; 
/*
PROC EXPORT DATA=filesave.AdvLong_Example6 OUTFILE= "&filesave.\AdvLong_Example6.xlsx" 
     DBMS=XLSX REPLACE; PUTNAMES=YES; RUN;
*/

* Open external file to save results to;
ODS RTF FILE="&filesave.\AdvLong_Example6_SAS_Output.rtf" 
        STYLE=HTMLBlue STARTPAGE=NO BODYTITLE; 

******************************************************************************;
******      PART 1: VARIANCE PARTITIONING AND SATURATED MEANS           ******;
******************************************************************************;

TITLE "SAS Model 1a: Empty Means, Two-Level Model for Cognition Outcome";
PROC MIXED DATA=work.Example6 NOCLPRINT COVTEST NAMELEN=100 IC METHOD=REML;
     CLASS PairID TwinID;
     MODEL info =  / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT / TYPE=UN SUBJECT=PairID*TwinID; * Level2+3 combined; 
     ODS OUTPUT InfoCrit=work.Fit2L; * Save fit stats for LRT; 
RUN; TITLE;

TITLE "SAS Model 1b: Empty Means, Three-Level Model for Cognition Outcome";
PROC MIXED DATA=work.Example6 NOCLPRINT COVTEST NAMELEN=100 IC METHOD=REML;
     CLASS PairID TwinID;
     MODEL info =   / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT / TYPE=UN SUBJECT=PairID;         * Level 3;
     RANDOM INTERCEPT / TYPE=UN SUBJECT=PairID*TwinID;  * Level 2;
     ODS OUTPUT InfoCrit=work.Fit3L CovParms=work.CovEmpty; * Save for LRT, pseudo-R2; 
RUN; TITLE;
* Compare three-level empty to two-level empty;
%FitTest(FitFewer=work.Fit2L, FitMore=work.Fit3L); 

TITLE "SAS Age Model: Empty Means, Three-Level Model for Age Predictor";
PROC MIXED DATA=work.Example6 NOCLPRINT COVTEST NAMELEN=100 IC METHOD=REML;
     CLASS PairID TwinID;
     MODEL age =   / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT / TYPE=UN SUBJECT=PairID;             * Level 3;
     RANDOM INTERCEPT / TYPE=UN SUBJECT=PairID*TwinID;      * Level 2;  
RUN; TITLE;

TITLE "SAS Model 1c: Saturated Wave Means, Random Intercepts for L2 Twin and L3 Pair";
PROC GLIMMIX DATA=work.Example6 NOCLPRINT NAMELEN=100 METHOD=RSPL; * Same as REML;
     CLASS PairID TwinID Wave;  * CLASS makes Wave a categorical predictor;
     MODEL info = Wave / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT / TYPE=UN SUBJECT=PairID;             * Level 3;
     RANDOM INTERCEPT / TYPE=UN SUBJECT=PairID*TwinID;      * Level 2; 
     LSMEANS Wave / PLOT=MEANPLOT(CLBAND JOIN);  * Print and plot saturated means; 
RUN; TITLE;

******************************************************************************;
******              PART 2: FIXED AND RANDOM EFFECTS OF TIME            ******;
******************************************************************************;

TITLE "SAS Model 2a: Fixed Quadratic, Random Intercepts for L2 Twin and L3 Pair";
PROC MIXED DATA=work.Example6 NOCLPRINT COVTEST NAMELEN=100 IC METHOD=REML;
     CLASS PairID TwinID;
     MODEL info = time time*time / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT / TYPE=UN SUBJECT=PairID;             * Level 3;
     RANDOM INTERCEPT / TYPE=UN SUBJECT=PairID*TwinID;      * Level 2; 
     ODS OUTPUT InfoCrit=work.FitRI2RI3 CovParms=work.CovFQuad; * Save for LRT, pseudo-R2; 
RUN; TITLE;
* Pseudo-R2 for linear and quadratic time;
%PseudoR2(Ncov=3, CovFewer=work.CovEmpty, CovMore=work.CovFQuad); 

TITLE "SAS Model 2b: Add Random Linear Time over L2 Twins";
PROC MIXED DATA=work.Example6 NOCLPRINT COVTEST NAMELEN=100 IC METHOD=REML;
     CLASS PairID TwinID;
     MODEL info = time time*time  / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT      /       TYPE=UN SUBJECT=PairID;         * Level 3;
     RANDOM INTERCEPT time / GCORR TYPE=UN SUBJECT=PairID*TwinID;  * Level 2; 
     ODS OUTPUT InfoCrit=work.FitRL2RI3; * Save for LRT, pseudo-R2; 
RUN; TITLE;
* Test random linear time over L2 twins;
%FitTest(FitFewer=work.FitRI2RI3, FitMore=work.FitRL2RI3); 

TITLE "SAS Model 2c: Add Random Linear Time over L3 Pairs";
PROC MIXED DATA=work.Example6 NOCLPRINT COVTEST NAMELEN=100 IC METHOD=REML;
     CLASS PairID TwinID;
     MODEL info = time time*time / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT time / GCORR TYPE=UN SUBJECT=PairID;        * Level 3;
     RANDOM INTERCEPT time / GCORR TYPE=UN SUBJECT=PairID*TwinID; * Level 2; 
     ODS OUTPUT InfoCrit=work.FitRL2RL3 CovParms=work.CovRL2RL3; * Save for LRT, pseudo-R2; 
RUN; TITLE;
* Test random linear time over L3 pairs;
%FitTest(FitFewer=work.FitRL2RI3, FitMore=work.FitRL2RL3); 

TITLE "SAS Model 2d: Add Random Quadratic Time over L2 Twins";
PROC MIXED DATA=work.Example6 NOCLPRINT COVTEST NAMELEN=100 IC METHOD=REML;
     CLASS PairID TwinID;
     MODEL info =  time time*time / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT time           / GCORR TYPE=UN SUBJECT=PairID;        * Level 3;
     RANDOM INTERCEPT time time*time / GCORR TYPE=UN SUBJECT=PairID*TwinID; * Level 2; 
     ODS OUTPUT InfoCrit=work.FitRQ2RL3; * Save for LRT, pseudo-R2; 
RUN; TITLE;
* Test random quadratic time over L2 twins;
%FitTest(FitFewer=work.FitRL2RL3, FitMore=work.FitRQ2RL3); 

TITLE "SAS Model 2e: Add Random Quadratic Time for L3 Pairs";
PROC MIXED DATA=work.Example6 NOCLPRINT COVTEST NAMELEN=100 IC METHOD=REML;
     CLASS PairID TwinID;
     MODEL info =  time time*time / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT time time*time / GCORR TYPE=UN SUBJECT=PairID;        * Level 3;
     RANDOM INTERCEPT time time*time / GCORR TYPE=UN SUBJECT=PairID*TwinID; * Level 2; 
     ODS OUTPUT InfoCrit=work.FitRQ2RQ3; * Save for LRT, pseudo-R2; 
RUN; TITLE;
* Test random quadratic time over L3 pairs;
%FitTest(FitFewer=work.FitRQ2RL3, FitMore=work.FitRQ2RQ3);

TITLE "SAS Model 3a: Add Baseline Age as Predictor of Pair Intercept and Change";
PROC MIXED DATA=work.Example6 NOCLPRINT COVTEST NAMELEN=100 IC METHOD=REML;
     CLASS PairID TwinID;
     MODEL info = time time*time PairAge85 time*PairAge85 time*time*PairAge85 
                  / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT time / GCORR TYPE=UN SUBJECT=PairID;        * Level 3;
     RANDOM INTERCEPT time / GCORR TYPE=UN SUBJECT=PairID*TwinID; * Level 2; 
     CONTRAST "Trajectory Diffs by Age" PairAge85 1, time*PairAge85 1, time*time*PairAge85 1;
     ODS OUTPUT InfoCrit=work.FitAge CovParms=work.CovAge; * Save for LRT, pseudo-R2; 
RUN; TITLE;
* Pseudo-R2 for age;
%PseudoR2(Ncov=7, CovFewer=work.CovRL2RL3, CovMore=work.CovAge); 

TITLE "SAS Model 3b: Add Zygosity as Predictor of Pair Intercept and Change";
PROC MIXED DATA=work.Example6 NOCLPRINT COVTEST NAMELEN=100 IC METHOD=REML;
     CLASS PairID TwinID zyg;
     MODEL info = time time*time PairAge85 time*PairAge85 time*time*PairAge85
                  IsDZ time*IsDZ time*time*IsDZ 
                  PairAge85*IsDZ time*PairAge85*IsDZ time*time*PairAge85*IsDZ
                  / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT time / GCORR TYPE=UN SUBJECT=PairID;        * Level 3;
     RANDOM INTERCEPT time / GCORR TYPE=UN SUBJECT=PairID*TwinID; * Level 2; 
     CONTRAST "Test of Overall Trajectory Diffs by Zygosity" 
               IsDZ 1, time*IsDZ 1, time*time*IsDZ 1, PairAge85*IsDZ 1, 
               time*PairAge85*IsDZ 1, time*time*PairAge85*IsDZ 1;
     ODS OUTPUT InfoCrit=work.FitZyg CovParms=work.CovZyg; * Save for LRT, pseudo-R2; 
RUN; TITLE;
* Pseudo-R2 for zygosity;
%PseudoR2(Ncov=7, CovFewer=work.CovAge, CovMore=work.CovZyg); 

TITLE "SAS Model 3c: Add Heterogeneous G and R matrices by Zygosity";
PROC MIXED DATA=work.Example6 NOCLPRINT COVTEST NAMELEN=100 IC METHOD=REML;
     CLASS PairID TwinID zyg;
     MODEL info = time time*time PairAge85 time*PairAge85 
                  IsDZ time*IsDZ time*time*IsDZ 
                  PairAge85*IsDZ time*PairAge85*IsDZ time*time*PairAge85*IsDZ 
                  / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT time / TYPE=UN SUBJECT=PairID        GROUP=zyg; * Level 3;
     RANDOM INTERCEPT time / TYPE=UN SUBJECT=PairID*TwinID GROUP=zyg; * Level 2; 
     REPEATED / GROUP=zyg; * Level 1 residual variance;
     ODS OUTPUT InfoCrit=work.FitHet CovParms=work.CovHet; * Save for LRT, pseudo-R2; 
RUN; TITLE;
* Test het variances by zygosity;
%FitTest(FitFewer=work.FitZyg, FitMore=work.FitHet); 

* Close external file of saved results;
ODS RTF CLOSE;


*********************************************************************************;
* Trimming data to send just needed variables to Mplus;
DATA work.Example6Mplus; * RETAIN re-orders variables as listed;
     RETAIN PairID TwinID wave info time PairAge85;
     SET work.Example6;
     * Telling it which variables to keep -- handy to use this in Mplus;
     KEEP PairID TwinID wave info time PairAge85;
     * Replace any missing values with -999;
     ARRAY avars(6) PairID TwinID wave info time PairAge85;
           DO i=1 TO 6; IF avars(i)=. THEN avars(i)=-999; END; DROP i; 
RUN;

* Export to .csv for use in Mplus MLM syntax;
PROC EXPORT DATA=work.Example6Mplus OUTFILE= "&filesave.\AdvLong_Example6.csv" 
     DBMS=CSV REPLACE; PUTNAMES=NO; RUN;


